# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0013_wooeyjob_uuid_populate'),
    ]

    operations = [
        # Set to unique=True
        migrations.AlterField(
            model_name='wooeyjob',
            name='uuid',
            field=models.CharField(default=uuid.uuid4, unique=True, max_length=255),
        ),
    ]



from . import test_settings
from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static

# the DEBUG setting in test_settings is not respected
settings.DEBUG = True


urlpatterns = [
    url(r'^', include('wooey.urls')),
    url(r'^', include('django.contrib.auth.urls')),
]

urlpatterns += static(test_settings.MEDIA_URL, document_root=test_settings.MEDIA_ROOT)
urlpatterns += static(test_settings.STATIC_URL, document_root=test_settings.STATIC_ROOT)



from __future__ import absolute_import

from django.conf.urls import include, url
from django.conf import settings
from django.conf.urls.static import static

from . import views
from . import settings as wooey_settings

app_name = 'wooey'

wooey_patterns = [
    url(r'^jobs/command$', views.celery_task_command, name='celery_task_command'),

    url(r'^jobs/queue/global/json$', views.global_queue_json, name='global_queue_json'),
    url(r'^jobs/queue/user/json$', views.user_queue_json, name='user_queue_json'),
    url(r'^jobs/results/user/json$', views.user_results_json, name='user_results_json'),

    url(r'^jobs/queue/all/json', views.all_queues_json, name='all_queues_json'),

    url(r'^jobs/queue/global', views.GlobalQueueView.as_view(), name='global_queue'),
    url(r'^jobs/queue/user', views.UserQueueView.as_view(), name='user_queue'),
    url(r'^jobs/results/user', views.UserResultsView.as_view(), name='user_results'),


    url(r'^jobs/(?P<job_id>[0-9\-]+)/$', views.JobView.as_view(), name='celery_results'),
    url(r'^jobs/(?P<job_id>[0-9\-]+)/json$', views.JobJSON.as_view(), name='celery_results_json'),
    url(r'^jobs/(?P<job_id>[0-9\-]+)/jsonhtml$', views.JobJSONHTML.as_view(), name='celery_results_json_html'),

    # Global public access via uuid
    url(r'^jobs/(?P<uuid>[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89aAbB][a-f0-9]{3}-[a-f0-9]{12})/$', views.JobView.as_view(), name='celery_results_uuid'),
    url(r'^jobs/(?P<uuid>[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89aAbB][a-f0-9]{3}-[a-f0-9]{12})/json$', views.JobJSON.as_view(), name='celery_results_json_uuid'),


    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/$', views.WooeyScriptView.as_view(), name='wooey_script'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/version/(?P<script_version>[^/]+)$', views.WooeyScriptView.as_view(), name='wooey_script'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/version/(?P<script_version>[^/]+)/iteration/(?P<script_iteration>\d+)$', views.WooeyScriptView.as_view(), name='wooey_script'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/version/(?P<script_version>[^/]+)/jobs/(?P<job_id>[a-zA-Z0-9\-]+)$', views.WooeyScriptView.as_view(), name='wooey_script'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/version/(?P<script_version>[^/]+)/iteration/(?P<script_iteration>\d+)/jobs/(?P<job_id>[a-zA-Z0-9\-]+)$', views.WooeyScriptView.as_view(), name='wooey_script'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/jobs/(?P<job_id>[a-zA-Z0-9\-]+)$', views.WooeyScriptView.as_view(), name='wooey_script_clone'),
    url(r'^scripts/(?P<slug>[a-zA-Z0-9\-\_]+)/$', views.WooeyScriptJSON.as_view(), name='wooey_script_json'),

    url(r'^scripts/search/json$', views.WooeyScriptSearchJSON.as_view(), name='wooey_search_script_json'),
    url(r'^scripts/search/jsonhtml$', views.WooeyScriptSearchJSONHTML.as_view(), name='wooey_search_script_jsonhtml'),


    url(r'^profile/$', views.WooeyProfileView.as_view(), name='profile_home'),
    url(r'^profile/(?P<username>[a-zA-Z0-9\-]+)$', views.WooeyProfileView.as_view(), name='profile'),

    url(r'^$', views.WooeyHomeView.as_view(), name='wooey_home'),
    url(r'^$', views.WooeyHomeView.as_view(), name='wooey_job_launcher'),
    url('^{}'.format(wooey_settings.WOOEY_LOGIN_URL.lstrip('/')), views.wooey_login, name='wooey_login'),

    url(r'^favorite/toggle$', views.toggle_favorite, name='toggle_favorite'),

    url(r'^scrapbook$', views.WooeyScrapbookView.as_view(), name='scrapbook'),


    url(r'^i18n/', include('django.conf.urls.i18n'), name='set_language'),

]

if wooey_settings.WOOEY_REGISTER_URL:
    wooey_patterns += [
        url(r'^{}'.format(wooey_settings.WOOEY_REGISTER_URL.lstrip('/')), views.WooeyRegister.as_view(), name='wooey_register'),
    ]

urlpatterns = [
    url(r'^', include(wooey_patterns)),
]



from __future__ import absolute_import, unicode_literals
from collections import defaultdict

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.forms import FileField
from django.http import JsonResponse
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from django.views.generic import DetailView, TemplateView, View

from ..backend import utils
from ..django_compat import reverse
from ..models import WooeyJob, Script, UserFile, Favorite, ScriptVersion
from .. import settings as wooey_settings


class WooeyScriptBase(DetailView):
    model = Script
    slug_field = 'slug'
    slug_url_kwarg = 'slug'

    @staticmethod
    def render_fn(s):
        return s

    def get_context_data(self, **kwargs):
        context = super(WooeyScriptBase, self).get_context_data(**kwargs)
        version = self.kwargs.get('script_version')
        iteration = self.kwargs.get('script_iteration')

        # returns the models required and optional fields as html
        job_id = self.kwargs.get('job_id')
        initial = defaultdict(list)

        if job_id:
            job = WooeyJob.objects.get(pk=job_id)
            if job.user is None or (self.request.user.is_authenticated and job.user == self.request.user):
                context['job_info'] = {'job_id': job_id}

                parser_used = None
                for i in job.get_parameters():
                    value = i.value
                    if value is not None:
                        script_parameter = i.parameter
                        if script_parameter.parser.name:
                            parser_used = script_parameter.parser.pk
                        initial[script_parameter.form_slug].append(value)

                if parser_used is not None:
                    initial['wooey_parser'] = parser_used

        script_version = ScriptVersion.objects.filter(
            script=self.object,
        )
        if not (version or iteration):
            script_version = script_version.get(default_version=True)
        else:
            if version:
                script_version = script_version.filter(script_version=version)
            if iteration:
                script_version = script_version.filter(script_iteration=iteration)

            script_version = script_version.order_by('script_version', 'script_iteration').last()

        context['form'] = utils.get_form_groups(
            script_version=script_version,
            initial_dict=initial,
            render_fn=self.render_fn,
        )
        return context

    def post(self, request, *args, **kwargs):
        post = request.POST.copy()
        user = request.user if request.user.is_authenticated else None
        if not wooey_settings.WOOEY_ALLOW_ANONYMOUS and user is None:
            return {'valid': False, 'errors': {'__all__': [force_text(_('You are not permitted to access this script.'))]}}

        form = utils.get_master_form(pk=int(post['wooey_type']), parser=int(post.get('wooey_parser', 0)))
        # TODO: Check with people who know more if there's a smarter way to do this
        utils.validate_form(form=form, data=post, files=request.FILES)
        # for cloned jobs, we don't have the files in input fields, they'll be in a list like ['', filename]
        # This will cause issues.
        to_delete = []
        for i in post:
            if isinstance(form.fields.get(i), FileField):
                # if we have a value set, reassert this
                new_values = list(filter(lambda x: x, post.getlist(i)))
                cleaned_values = []
                for new_value in new_values:
                    if i not in request.FILES and (i not in form.cleaned_data or (new_value and (form.cleaned_data[i] is None or not [j for j in form.cleaned_data[i] if j]))):
                        # this is a previously set field, so a cloned job
                        if new_value is not None:
                            cleaned_values.append(utils.get_storage(local=False).open(new_value))
                        to_delete.append(i)
                if cleaned_values:
                    form.cleaned_data[i] = cleaned_values
        for i in to_delete:
            if i in form.errors:
                del form.errors[i]

        # because we can have multiple files for a field, we need to update our form.cleaned_data to be a list of files
        for i in request.FILES:
            v = request.FILES.getlist(i)
            if i in form.cleaned_data:
                cleaned = form.cleaned_data[i]
                cleaned = cleaned if isinstance(cleaned, list) else [cleaned]
                form.cleaned_data[i] = list(set(cleaned).union(set(v)))

        if not form.errors:
            version_pk = form.cleaned_data.get('wooey_type')
            parser_pk = form.cleaned_data.get('wooey_parser')
            script_version = ScriptVersion.objects.get(pk=version_pk)
            valid = utils.valid_user(script_version.script, request.user).get('valid')
            if valid == True:
                group_valid = utils.valid_user(script_version.script.script_group, request.user).get('valid')
                if valid == True and group_valid == True:
                    job = utils.create_wooey_job(
                        script_parser_pk=parser_pk,
                        script_version_pk=version_pk,
                        user=user,
                        data=form.cleaned_data
                    )
                    job.submit_to_celery()
                    return {'valid': True, 'job_id': job.id}

            return {'valid': False, 'errors': {'__all__': [force_text(_('You are not permitted to access this script.'))]}}

        return {'valid': False, 'errors': form.errors}


class WooeyScriptJSON(WooeyScriptBase):

    # FIXME: the form data is returned as form objects so can be passed to templates
    # this render_fn allows us to pass the return through a stringify method for JSON
    @staticmethod
    def render_fn(form):
        return form.as_table()

    def render_to_response(self, context, *args, **kwargs):
        return JsonResponse(context)

    def post(self, *args, **kwargs):
        data = super(WooeyScriptJSON, self).post(*args, **kwargs)
        return JsonResponse(data)


class WooeyScriptView(WooeyScriptBase):

    template_name = 'wooey/scripts/script_view.html'

    def post(self, *args, **kwargs):
        data = super(WooeyScriptView, self).post(*args, **kwargs)
        if data['valid']:
            data['redirect'] = reverse('wooey:celery_results', kwargs={'job_id': data['job_id']})
        return JsonResponse(data)


class WooeyHomeView(TemplateView):
    template_name = 'wooey/home.html'

    def get_context_data(self, **kwargs):
        #job_id = self.request.GET.get('job_id')
        ctx = super(WooeyHomeView, self).get_context_data(**kwargs)
        ctx['scripts'] = utils.get_current_scripts()

        # Check for logged in user
        if self.request.user.is_authenticated:
            # Get the id of every favorite (scrapbook) file
            ctype = ContentType.objects.get_for_model(Script)
            ctx['favorite_script_ids'] = Favorite.objects.filter(content_type=ctype, user__id=self.request.user.id).values_list('object_id', flat=True)
        else:
            ctx['favorite_script_ids'] = []

        return ctx


class WooeyProfileView(TemplateView):
    template_name = 'wooey/profile/profile.html'

    def get_context_data(self, **kwargs):
        ctx = super(WooeyProfileView, self).get_context_data(**kwargs)

        if 'username' in self.kwargs:
            user = get_user_model()
            ctx['profile_user'] = user.objects.get(username=self.kwargs.get('username'))

        else:
            if self.request.user and self.request.user.is_authenticated:
                ctx['profile_user'] = self.request.user

        return ctx


class WooeyScrapbookView(TemplateView):
    template_name = 'wooey/scrapbook.html'

    def get_context_data(self, **kwargs):
        ctx = super(WooeyScrapbookView, self).get_context_data(**kwargs)

        # Get the id of every favorite (scrapbook) file
        ctype = ContentType.objects.get_for_model(UserFile)
        favorite_file_ids = Favorite.objects.filter(content_type=ctype, user=self.request.user).values_list('object_id', flat=True)

        out_files = utils.get_file_previews_by_ids(favorite_file_ids)

        all = out_files.pop('all', [])
        archives = out_files.pop('archives', [])

        ctx['file_groups'] = out_files
        ctx['favorite_file_ids'] = favorite_file_ids

        return ctx


class WooeySearchBase(View):

    model = None
    search_fields = []

    def get(self, request, *args, **kwargs):

        self.search_results = None
        if 'q' in request.GET:
            query_string = request.GET['q'].strip()

            query = utils.get_query(query_string, self.search_fields)
            self.search_results = self.model.objects.filter(query)

            return self.search(request, *args, **kwargs)


class WooeyScriptSearchBase(WooeySearchBase):

    model = Script
    search_fields = ['script_name', 'script_description']


class WooeyScriptSearchJSON(WooeyScriptSearchBase):
    """
    Returns the result of the script search as JSON containing data only
    """

    def search(self, request):
        results = []
        for script in self.search_results:
            results.append({
                'id': script.id,
                'name': script.script_name,
                'description': script.script_description,
                'url': reverse('wooey:wooey_script', kwargs={'slug': script.slug}),
            })
        return JsonResponse({'results': results})


class WooeyScriptSearchJSONHTML(WooeyScriptSearchBase):
    """
    Returns the result of the script search as JSON containing rendered template
    elements for display in the original page. This is a temporary function
    until the handling is moved to client side rendering.
    """

    def search(self, request):
        results = []
        for script in self.search_results:
            results.append(render_to_string('wooey/scripts/script_panel.html', {'script': script}, context_instance=RequestContext(request)))
        return JsonResponse({'results': results})



from django.db import models
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from ..version import DJANGO_VERSION, DJ18

# This cannot be in django_compat due to the other modules which import it prior to application loading.
# We can think about later adding a django_compat_post_load or something to handle these corner cases if more arise
# GenericForeignKey compatibility fixes
if DJANGO_VERSION < DJ18:
    from django.contrib.contenttypes.generic import GenericForeignKey

else:
    from django.contrib.contenttypes.fields import GenericForeignKey


class Favorite(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='favorites', blank=True, null=True, on_delete=models.CASCADE)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    created_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'content_type', 'object_id', )
        app_label = 'wooey'
        verbose_name = _('favorite')
        verbose_name_plural = _('favorites')



__author__ = 'chris'
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


def get(key, default):
    return getattr(settings, key, default)

# AUTH based settings
WOOEY_ALLOW_ANONYMOUS = get('WOOEY_ALLOW_ANONYMOUS', True)
WOOEY_AUTH = get('WOOEY_AUTH', True)
WOOEY_LOGIN_URL = get('WOOEY_LOGIN_URL', settings.LOGIN_URL)
WOOEY_REGISTER_URL = get('WOOEY_REGISTER_URL', '/accounts/register/')

# Celery and job queue settings
WOOEY_CELERY = get('WOOEY_CELERY', True)
WOOEY_CELERY_TASKS = get('WOOEY_CELERY_TASKS', 'wooey.tasks')
WOOEY_CELERY_STOPPABLE_JOBS = 'amqp' in get('CELERY_BROKER_URL', '')

# Site setup settings
WOOEY_DEFAULT_SCRIPT_GROUP = get('WOOEY_DEFAULT_SCRIPT_GROUP', _('Scripts'))
WOOEY_EPHEMERAL_FILES = get('WOOEY_EPHEMERAL_FILES', False)
WOOEY_FILE_DIR = get('WOOEY_FILE_DIR', 'wooey_files')
WOOEY_JOB_EXPIRATION = get('WOOEY_JOB_EXPIRATION', {'anonymous': None, 'users': None})
WOOEY_REALTIME_CACHE = get('WOOEY_REALTIME_CACHE', None)
WOOEY_SCRIPT_DIR = get('WOOEY_SCRIPT_DIR', 'wooey_scripts')

# User interface settings
WOOEY_SHOW_LOCKED_SCRIPTS = get('WOOEY_SHOW_LOCKED_SCRIPTS', True)
WOOEY_SITE_NAME = get('WOOEY_SITE_NAME', _('Wooey!'))
WOOEY_SITE_TAG = get('WOOEY_SITE_TAG', _('A web UI for Python scripts'))




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0011_script_versioning_cleanup'),
    ]

    operations = [
        # Add the uuid field with unique=False for existing entries
        # due to a bug in migrations this will set all to the same uuid
        migrations.AddField(
            model_name='wooeyjob',
            name='uuid',
            field=models.CharField(default=uuid.uuid4, unique=False, max_length=255),
        ),
    ]



#!/usr/bin/env python
__author__ = 'chris'
import argparse
import sys

# This script just exists to force a checksum difference from translate.py

BASE_PAIR_COMPLEMENTS = {'a': 't', 't': 'a', 'c': 'g', 'g': 'c', 'A': 'T', 'T': 'A', 'C': 'G', 'G': 'C', 'n': 'n', 'N': 'N'}

CODON_TABLE = {'AAA': 'K', 'AAC': 'N', 'AAG': 'K', 'AAT': 'N', 'ACA': 'T', 'ACC': 'T', 'ACG': 'T', 'ACT': 'T',
               'AGA': 'R', 'AGC': 'S', 'AGG': 'R', 'AGT': 'S', 'ATA': 'I', 'ATC': 'I', 'ATG': 'M', 'ATT': 'I',
               'CAA': 'Q', 'CAC': 'H', 'CAG': 'Q', 'CAT': 'H', 'CCA': 'P', 'CCC': 'P', 'CCG': 'P', 'CCT': 'P',
               'CGA': 'R', 'CGC': 'R', 'CGG': 'R', 'CGT': 'R', 'CTA': 'L', 'CTC': 'L', 'CTG': 'L', 'CTT': 'L',
               'GAA': 'E', 'GAC': 'D', 'GAG': 'E', 'GAT': 'D', 'GCA': 'A', 'GCC': 'A', 'GCG': 'A', 'GCT': 'A',
               'GGA': 'G', 'GGC': 'G', 'GGG': 'G', 'GGT': 'G', 'GTA': 'V', 'GTC': 'V', 'GTG': 'V', 'GTT': 'V',
               'TAA': '*', 'TAC': 'Y', 'TAG': '*', 'TAT': 'Y', 'TCA': 'S', 'TCC': 'S', 'TCG': 'S', 'TCT': 'S',
               'TGA': '*', 'TGC': 'C', 'TGG': 'W', 'TGT': 'C', 'TTA': 'L', 'TTC': 'F', 'TTG': 'L', 'TTT': 'F',
               'NNN': 'X'}

for i in 'ACTG':
    for j in 'ACTG':
        CODON_TABLE['%s%sN' % (i, j)] = 'X'
        CODON_TABLE['%sN%s' % (i, j)] = 'X'
        CODON_TABLE['N%s%s' % (i, j)] = 'X'
    CODON_TABLE['%sNN' % i] = 'X'
    CODON_TABLE['N%sN' % i] = 'X'
    CODON_TABLE['NN%s' % i] = 'X'

parser = argparse.ArgumentParser(description="This will translate a given DNA sequence to protein.")
group = parser.add_mutually_exclusive_group(required=True)
group.add_argument('--sequence', help='The sequence to translate.', type=str)
group.add_argument('--fasta', help='The fasta file to translate.', type=argparse.FileType('rb'))
simple_group = parser.add_argument_group('Parameter Group')
simple_group.add_argument('--frame', help='The frame to translate in.', type=str, choices=['+1', '+2', '+3', '-1', '-2', '-3'], default='+1')
simple_group.add_argument('--out', help='The file to save translations to.', type=argparse.FileType('wb'))


def main():
    args = parser.parse_args()
    seq = args.sequence
    fasta = args.fasta

    def translate(seq=None, frame=None):
        if frame.startswith('-'):
            seq = ''.join([BASE_PAIR_COMPLEMENTS.get(i, 'N') for i in seq])
        frame = int(frame[1])-1
        return ''.join([CODON_TABLE.get(seq[i:i+3], 'X') for i in xrange(frame, len(seq), 3) if i+3<=len(seq)])

    frame = args.frame
    with args.out as fasta_out:
        if fasta:
            with args.fasta as fasta_in:
                header = ''
                seq = ''
                for row in fasta_in:
                    if row[0] == '>':
                        if seq:
                            fasta_out.write('{}\n{}\n'.format(header, translate(seq, frame)))
                        header = row
                        seq = ''
                    else:
                        seq += row.strip()
                if seq:
                    fasta_out.write('{}\n{}\n'.format(header, translate(seq, frame)))
        else:
            fasta_out.write('{}\n{}\n'.format('>1', translate(seq.upper(), frame)))

if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .mixins import WooeyPy2Mixin
from ..backend import utils

class WooeyWidget(WooeyPy2Mixin, models.Model):
    name = models.CharField(_('Widget Name'), max_length=50)
    widget_class = models.CharField(
        max_length=50,
        help_text=_('Widget class to use (e.g. django.forms.TextInput, defaults to Form Field on Script Parameter model if blank).'),
        null=True,
        blank=True,
    )
    input_attributes = models.TextField(
        verbose_name=_('Input Widget Extra Attributes'),
        help_text=_('Extra attributes to the input field. The extra attributes MUST be specified like key="value" (e.g. type="date").'),
        null=True,
        blank=True,
    )
    input_class = models.CharField(
        max_length=255,
        verbose_name=_('Input Widget Class name(s)'),
        help_text=_('The class name(s) for the input field.'),
        null=True,
        blank=True,
    )
    input_properties = models.CharField(
        max_length=255,
        verbose_name=_('Input Widget Extra Properties'),
        help_text=_('Additional properties to append to the input field.'),
        null=True,
        blank=True,
    )


    @property
    def widget_attributes(self):
        attrs = {}

        properties = self.input_properties
        if properties is not None:
            for attr in properties.split(' '):
                attrs[attr] = True

        attributes = self.input_attributes
        if attributes is not None:
            for key, value in utils.tokenize_html_attributes(attributes):
                attrs[key] = value

        if self.input_class:
            attrs['class'] = self.input_class

        return attrs

    def __str__(self):
        return self.name


from django.template import Context
from django.test import TestCase

from ..django_compat import get_template_from_string
from ..templatetags import wooey_tags
from .. import settings as wooey_settings
from .factories import UserFactory


class TemplateTagsTestCase(TestCase):

    def test_get_wooey_setting(self):
        #test that get_wooey_setting works as expected
        self.assertEqual(wooey_tags.get_wooey_setting("WOOEY_SITE_NAME"), wooey_settings.WOOEY_SITE_NAME)
        #test that get_wooey_setting works following a change
        wooey_settings.WOOEY_SITE_NAME = "TEST_SITE"
        self.assertEqual(wooey_tags.get_wooey_setting("WOOEY_SITE_NAME"), wooey_settings.WOOEY_SITE_NAME)


    def test_gravatar(self):
        t = get_template_from_string("{% load wooey_tags %}{% gravatar user.email 64 %}")
        user = UserFactory()
        self.assertEqual(
            t.render(Context({'user': user})),
            "http://www.gravatar.com/avatar/d10ca8d11301c2f4993ac2279ce4b930?s=64"
        )
        


import os
import six
import zipfile

from django.test import TestCase, TransactionTestCase

from ..backend import utils

from . import (
    config,
    factories,
    mixins,
    utils as test_utils,
)


class TestUtils(mixins.ScriptFactoryMixin, mixins.FileMixin, TransactionTestCase):
    def test_sanitize_name(self):
        assert(utils.sanitize_name('abc')) == 'abc'
        assert(utils.sanitize_name('ab c')) == 'ab_c'
        assert(utils.sanitize_name('ab-c')) == 'ab_c'

    def test_sanitize_string(self):
        assert(utils.sanitize_string('ab"c')) == 'ab\\"c'

    def test_anonymous_users(self):
        from .. import settings as wooey_settings
        from django.contrib.auth.models import AnonymousUser
        user = AnonymousUser()
        script_version = self.translate_script
        script = script_version.script
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])
        wooey_settings.WOOEY_ALLOW_ANONYMOUS = False
        d = utils.valid_user(script, user)
        self.assertFalse(d['valid'])

    def test_valid_user(self):
        user = factories.UserFactory()
        script_version = self.translate_script
        script = script_version.script
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])
        from .. import settings as wooey_settings
        self.assertEqual('disabled', d['display'])
        wooey_settings.WOOEY_SHOW_LOCKED_SCRIPTS = False
        d = utils.valid_user(script, user)
        self.assertEqual('hide', d['display'])
        from django.contrib.auth.models import Group
        test_group = Group(name='test')
        test_group.save()
        script.user_groups.add(test_group)
        d = utils.valid_user(script, user)
        self.assertFalse(d['valid'])
        user.groups.add(test_group)
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])

    def test_job_file_outputs(self):
        # Run a script that creates a file
        from wooey.models import WooeyJob, UserFile
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'file_maker.py')
        with open(script) as o:
            new_file = self.storage.save(self.filename_func('file_maker.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])

        output_slug = test_utils.get_subparser_form_slug(res['script'], 'output')
        job = utils.create_wooey_job(
            script_version_pk=res['script'].pk,
            data={
                'job_name': 'abc',
                output_slug: 'test_file',
            },
        )
        # Get the new job
        job.submit_to_celery()
        job = WooeyJob.objects.get(pk=job.pk)

        # Make sure the file info is correct
        self.assertEqual(UserFile.objects.filter(job=job).count(), 4)
        for job_file in UserFile.objects.filter(job=job):
            wooey_file = job_file.system_file
            self.assertEqual(
                os.path.getsize(self.storage.path(wooey_file.filepath.name)),
                wooey_file.size_bytes,
            )

        # Check for the zip file overwrite in for https://github.com/wooey/Wooey/issues/202
        zip_file = UserFile.objects.get(job=job, filename__endswith='zip')
        _zip = zipfile.ZipFile(zip_file.system_file.filepath)
        files = [filename.filename for filename in _zip.filelist]
        six.assertCountEqual(self, files, ['abc/', 'abc/test_file', 'abc/test_dir/test_file'])

    def test_duplicate_scriptversion_checksums(self):
        # Tests that script versions with duplicate checksums are correctly returned
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v1.py')
        with open(script) as o:
            v1 = self.storage.save(self.filename_func('v1.py'), o)
        res = utils.add_wooey_script(script_path=v1, script_name='test_versions')
        first_version = res['script']
        first_version_pk = first_version.pk
        self.assertEqual(res['valid'], True, res['errors'])

        # Force a duplication of the Script Version, this mimics legacy code that may
        # have several script versions that were uploaded.

        first_version.pk = None
        first_version.script_iteration = first_version.script_iteration + 1
        first_version.save()
        second_version_pk = first_version.pk

        # Add the script again, which should not fail on MultipleObjectsReturned
        res = utils.add_wooey_script(script_path=v1, script_name='test_versions')
        self.assertEqual(res['script'].pk, second_version_pk)

    def test_add_wooey_script(self):
        from wooey.models import ScriptParser, ScriptVersion
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v1.py')
        with open(script) as o:
            v1 = self.storage.save(self.filename_func('v1.py'), o)
        res = utils.add_wooey_script(script_path=v1, script_name='test_versions')
        first_version = res['script']
        self.assertEqual(res['valid'], True, res['errors'])
        parser = ScriptParser.objects.get(script_version=res['script'])
        self.assertEqual(parser.name, u'')

        # Test that adding the script again doesn't update anything
        res = utils.add_wooey_script(script_path=v1, script_name='test_versions')
        self.assertEqual(res['valid'], False)
        self.assertEqual(str(res['errors']), ScriptVersion.error_messages['duplicate_script'])
        dup_first_version = res['script']
        self.assertEqual(
            first_version.script_version,
            dup_first_version.script_version
        )
        self.assertEqual(first_version.script_iteration, 1)
        self.assertEqual(dup_first_version.script_iteration, 1)

        # Test updating to script2 keeps the same reference to the --one parameter
        # between scripts
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v2.py')
        with open(script) as o:
            v2 = self.storage.save(self.filename_func('v2.py'), o)

        res = utils.add_wooey_script(script_path=v2, script_name='test_versions')
        second_version = res['script']
        first_params = {i.pk for i in first_version.get_parameters()}
        second_params = {i.pk for i in second_version.get_parameters()}
        self.assertTrue(first_params.issubset(second_params))
        self.assertTrue(len(second_params), 2)

        # update to v3, drops the second param
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'versioned_script', 'v3.py')
        with open(script) as o:
            v3 = self.storage.save(self.filename_func('v3.py'), o)

        res = utils.add_wooey_script(script_path=v3, script_name='test_versions')
        third_version = res['script']
        third_params = {i.pk for i in third_version.get_parameters()}
        self.assertEqual(first_params, third_params)


class TestFileDetectors(TestCase):
    def test_detector(self):
        self.file = os.path.join(config.WOOEY_TEST_DATA, 'fasta.fasta')
        res, preview = utils.test_fastx(self.file)
        self.assertEqual(res, True, 'Fastx parser fail')
        self.assertEqual(preview, open(self.file).readlines(), 'Fastx Preview Fail')

    def test_delimited(self):
        self.file = os.path.join(config.WOOEY_TEST_DATA, 'delimited.tsv')
        res, preview = utils.test_delimited(self.file)
        self.assertEqual(res, True, 'Delimited parser fail')
        self.assertEqual(preview, [i.strip().split('\t') for i in open(self.file).readlines()], 'Delimited Preview Fail')



import mock
import os

from django.test import TransactionTestCase

from . import config, mixins, utils as test_utils
from ..backend import utils
from ..models import ScriptParameter, ScriptVersion


class ScriptAdditionTests(mixins.ScriptFactoryMixin, mixins.FileMixin, TransactionTestCase):

    def test_command_order(self):
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py')
        with open(script) as o:
            new_file = self.storage.save(self.filename_func('command_order.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        link_slug = test_utils.get_subparser_form_slug(res['script'], 'link')
        name_slug = test_utils.get_subparser_form_slug(res['script'], 'name')
        job = utils.create_wooey_job(
            script_version_pk=res['script'].pk,
            data={
                'job_name': 'abc',
                link_slug: 'alink',
                name_slug: 'aname'
            }
        )
        # These are positional arguments -- we DO NOT want them returning anything
        self.assertEqual(['', ''], [i.parameter.short_param for i in job.get_parameters()])
        # These are the params shown to the user, we want them returning their destination
        # This also checks that we maintain the expected order
        self.assertEqual(['link', 'name'], [i.parameter.script_param for i in job.get_parameters()])
        # Check the job command
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(['alink', 'aname'], commands)

    def test_default_parameters(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate.py')
        with open(script_path) as o:
            new_file = self.storage.save(self.filename_func('translate.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        script_version = res['script']
        script_parameter = script_version.scriptparameter_set.get(script_param='frame')
        self.assertEqual(script_parameter.default, '+1')

    def test_collapse_arguments(self):
        slug = test_utils.get_subparser_form_slug(self.choice_script, 'need_at_least_one_numbers')
        job = utils.create_wooey_job(
            script_version_pk=self.choice_script.pk,
            data={
                'job_name': 'abc',
                slug: [1, 2]
            }
        )
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(commands, ['--need-at-least-one-numbers', '1', '--need-at-least-one-numbers', '2'])

        slug = test_utils.get_subparser_form_slug(self.choice_script, 'choices_str')
        job = utils.create_wooey_job(
            script_version_pk=self.choice_script.pk,
            data={
                'job_name': 'abc',
                slug: [1, 2, 3]
            }
        )
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(commands, ['--choices-str', '1', '2', '3'])

    def test_script_upgrade(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate.py')
        with open(script_path) as o:
            new_file = self.storage.save(self.filename_func('translate.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        # upgrade script
        script = ScriptVersion.objects.get(pk=res['script'].pk)
        script_path2 = os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate2.py')
        with open(script_path2) as o:
            new_script = self.storage.save(self.filename_func('translate2.py'), o)
        script.script_path = new_script
        # we are going to be cloning this, so we lose the old object
        old_pk, old_iter = script.pk, script.script_iteration
        script.save()
        self.assertNotEqual(old_pk, script.pk)
        self.assertNotEqual(old_iter, script.script_iteration)
        # asset we are using the latest script in the frontend
        self.assertIn(script, [i.latest_version for i in utils.get_current_scripts()])
        old_script = ScriptVersion.objects.get(pk=old_pk)
        self.assertNotIn(old_script, [i.latest_version for i in utils.get_current_scripts()])

    def test_script_parameter_upgrade(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices.py')
        script_2_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices_2.py')

        # Upload the script
        with open(script_path) as o:
            new_file = self.storage.save(self.filename_func('choices.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])

        # upgrade script
        script = ScriptVersion.objects.get(pk=res['script'].pk)
        with open(script_2_path) as o:
            new_script = self.storage.save(self.filename_func('choices.py'), o)
        script.script_path = new_script
        # we are going to be cloning this, so we lose the old object and we keep a reference here
        old_pk, old_iter = script.pk, script.script_iteration
        script.save()

        self.assertNotEqual(old_pk, script.pk)
        self.assertNotEqual(old_iter, script.script_iteration)
        # asset we are using the latest script in the frontend
        self.assertIn(script, [i.latest_version for i in utils.get_current_scripts()])
        old_script = ScriptVersion.objects.get(pk=old_pk)
        self.assertNotIn(old_script, [i.latest_version for i in utils.get_current_scripts()])
        # Assert that the 'one_choice' parameter is different between the versions, but all else is the same
        old_parameters = list(old_script.get_parameters())
        new_parameters = list(script.get_parameters())
        self.assertListEqual(old_parameters[1:], new_parameters[1:])
        self.assertTrue(new_parameters[0].short_param, '--one-choice-added')
        self.assertTrue(old_parameters[0].short_param, '--one-choice')

    def test_redownloads_on_script_update(self):
        # Run the translate script, update it, and ensure the new version is downloaded
        sequence_slug = test_utils.get_subparser_form_slug(self.translate_script, 'sequence')
        job = utils.create_wooey_job(script_version_pk=self.translate_script.pk, data={'job_name': 'job1', sequence_slug: 'ATG'})
        job = job.submit_to_celery()

        # The file has been downloaded locally, update the script and ensure the new one is used
        script_path2 = os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate2.py')
        with open(script_path2) as o:
            new_script = self.storage.save(self.filename_func('translate2.py'), o)
        self.translate_script.script_path = new_script
        self.translate_script.save()

        job = utils.create_wooey_job(script_version_pk=self.translate_script.pk,
                                     data={'job_name': 'job1', sequence_slug: 'ATC'})
        with mock.patch('wooey.backend.utils.default_storage.local_storage.save') as storage_save:
            job.submit_to_celery()
            storage_save.assert_called()

    def test_reuses_script_if_not_updated(self):
        sequence_slug = test_utils.get_subparser_form_slug(self.translate_script, 'sequence')
        job = utils.create_wooey_job(script_version_pk=self.translate_script.pk,
                                     data={'job_name': 'job1', sequence_slug: 'ATG'})
        job.submit_to_celery()

        job = utils.create_wooey_job(script_version_pk=self.translate_script.pk,
                                     data={'job_name': 'job2', sequence_slug: 'ATC'})
        with mock.patch('wooey.backend.utils.default_storage.local_storage.save') as storage_save:
            job.submit_to_celery()
            storage_save.assert_not_called()


# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-15 15:43
from __future__ import unicode_literals

from django.db import migrations


def populate_m2m(apps, schema_editor):
    ScriptParser = apps.get_model('wooey', 'ScriptParser')
    ScriptParameterGroup = apps.get_model('wooey', 'ScriptParameterGroup')
    for obj in ScriptParser.objects.all():
        obj.new_script_version.add(obj.script_version)

    for obj in ScriptParameterGroup.objects.all():
        obj.new_script_version.add(obj.script_version)

class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0029_add-m2m-sv'),
    ]

    operations = [
        migrations.RunPython(populate_m2m)
    ]
